package com.diagbot.repository;

import com.diagbot.biz.push.entity.Lis;
import com.diagbot.dto.*;
import com.diagbot.entity.node.*;
import com.diagbot.util.NeoUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.vo.CriticalNeoVO;
import com.diagbot.vo.NeoEntityVO;

import java.math.BigDecimal;
import java.util.*;

public class LisNode {

	public LisBillNeoDTO ListoLISDTO(LIS lis) {

		LisBillNeoDTO lisBillNeoDTO = new LisBillNeoDTO();

		Gender gender = lis.getGender();
		if (null!=gender) {
			lisBillNeoDTO.setGender(NeoUtil.updateNodeInfo(gender.getName(), null, null, null, null));
		}


		Age age = lis.getAge();
		if (null!=age) {
			NodeNeoDTO ageNeoDTO = new NodeNeoDTO();
			ageNeoDTO.setMin(BigDecimal.valueOf(age.getMinval()));
			ageNeoDTO.setMax(BigDecimal.valueOf(age.getMaxval()));
			ageNeoDTO.setName("年龄");
			lisBillNeoDTO.setAgeNeoDTO(ageNeoDTO);
		}

		Set<ClinicalFinding> clinicalfindings = lis.getClinicalfindings();
		for (ClinicalFinding clinicalfinding : clinicalfindings) {
			String name = clinicalfinding.getName();
			BigDecimal min = NeoUtil.DoubletoBigDecimal(clinicalfinding.getMinval());
			BigDecimal max = NeoUtil.DoubletoBigDecimal(clinicalfinding.getMaxval());
			String unit = clinicalfinding.getUnit();
			Integer range = clinicalfinding.getRange();
			lisBillNeoDTO.getClinicfindings().add(NeoUtil.updateNodeInfo(name, min, max, unit, range));
		}

		Set<Vital> vitals = lis.getVitals();
		for (Vital vital : vitals) {
			String name = vital.getName();
			BigDecimal min = NeoUtil.DoubletoBigDecimal(vital.getMinval());
			BigDecimal max = NeoUtil.DoubletoBigDecimal(vital.getMaxval());
			String unit = vital.getUnit();
			Integer range = vital.getRange();
			lisBillNeoDTO.getVitals().add(NeoUtil.updateNodeInfo(name, min, max, unit, range));
		}

		Set<Medicine> conflict_medicines = lis.getConflict_medicines();
		for (Medicine conflict_medicine : conflict_medicines) {
			String name = conflict_medicine.getName();
			lisBillNeoDTO.getOralmeds().add(NeoUtil.updateNodeInfo(name, null, null, null, null));
		}

		Set<AllergicMed> allergicmeds = lis.getAllergicmeds();
		for (AllergicMed allergicmed : allergicmeds) {
			String name = allergicmed.getName();
			lisBillNeoDTO.getAllergicmeds().add(NeoUtil.updateNodeInfo(name, null, null, null, null));
		}

		Set<Operation> operations = lis.getOperations();
		for (Operation operation : operations) {
			String name = operation.getName();
			lisBillNeoDTO.getOperations().add(NeoUtil.updateNodeInfo(name, null, null, null, null));
		}

		Set<Procedure> procedures = lis.getProcedures();
		for (Procedure procedure : procedures) {
			String name = procedure.getName();
			lisBillNeoDTO.getProcedures().add(NeoUtil.updateNodeInfo(name, null, null, null, null));
		}

		Set<PACS> pacss = lis.getPacs();
		for (PACS pacs : pacss) {
			String name = pacs.getName();
//			BigDecimal min = pacs.getMinval();
//			BigDecimal max = pacs.getMaxval();
//			String unit = pacs.getUnit();
//			lisBillNeoDTO.getPacs().add(NeoUtil.updateNodeInfo(name, min, max, unit));
			lisBillNeoDTO.getPacs().add(NeoUtil.updateNodeInfo(name, null, null, null, null));
		}

		Set<Disease> diseases = lis.getDiseases();
		for (Disease disease : diseases) {
			String name = disease.getName();
			lisBillNeoDTO.getDisease().add(NeoUtil.updateNodeInfo(name, null, null, null, null));
		}

		Set<Group> groups = lis.getGroups();
		for (Group group : groups) {
			String name = group.getName();
			lisBillNeoDTO.getGroup().add(NeoUtil.updateNodeInfo(name, null, null, null, null));
		}


		return lisBillNeoDTO;
	}


	/**
	 * 处理化验开单合理性
	 */
	public List<BillNeoDTO> getLisDetailBill(Map<String, String> listerm, LisRepository lisRepository) {
		List<BillNeoDTO> billNeoDTOs = new ArrayList<>();
		BillNeoDTO billNeoDTO;

		LisBillNeoDTO lisBillNeoDTO;

		LIS lis;
		String lisname;

		for (String term : listerm.keySet()) {
			billNeoDTO = new BillNeoDTO();
			lisname = listerm.get(term);
			billNeoDTO.setName(term);
			billNeoDTO.setStandname(lisname);

			lisBillNeoDTO = new LisBillNeoDTO();

			lis = lisRepository.findByNameIs(lisname).get(0);

			if (lis != null) {
				lisBillNeoDTO = ListoLISDTO(lis);
			}

			billNeoDTO.setLisBillNeoDTO(lisBillNeoDTO);

			billNeoDTOs.add(billNeoDTO);
		}

		return billNeoDTOs;
	}



	/**
	 * 获取化验细项名称列表
	 */
	public List<LisDetailDTO> getLisDetailList(String name, LisRepository lisRepository) {
		String nodename;

		List<LisDetailDTO> lislist = new ArrayList<>();
		LisDetailDTO lisDetailDTO;

		List<LIS> lislt = lisRepository.findByNameLike("*"+name+"*");

		for (LIS lis: lislt) {
			lisDetailDTO = new LisDetailDTO();
			nodename = lis.getName();
			lisDetailDTO.setUniqueName(nodename);

			if (lis.getLisPacks_related_lis().size()>0) {
				List<LISPack> lispacklist = new ArrayList<>(lis.getLisPacks_related_lis());
				for (LISPack lisPack : lispacklist) {
					lisDetailDTO = new LisDetailDTO();
					nodename = lis.getName();
					lisDetailDTO.setUniqueName(nodename);
					lisDetailDTO.setName(lisPack.getName());
					lislist.add(lisDetailDTO);
				}
			}
			else {
				lisDetailDTO.setName(nodename+"(套餐)");
				lislist.add(lisDetailDTO);
			}
		}


		return lislist;
	}



	/**
	 * 获取化验细项危急值
	 */
	public List<CriticalNeoDTO> getLisCritialList(CriticalNeoVO criticalNeoVO, LisRepository lisRepository) {
		String nodename;
		Integer agemin, agemax;

		List<CriticalNeoDTO> lislist = new ArrayList<>();
		CriticalNeoDTO criticalNeoDTO;

		Integer gender = criticalNeoVO.getSex();
		Integer age = criticalNeoVO.getAge();
		List<Lis> lisVO = criticalNeoVO.getLis();

		try {
			for (Lis lisvo : lisVO) {

				String name = lisvo.getDetailName();

				if (StringUtil.isNotBlank(name)) {
					List<LIS> lislt = lisRepository.findByNameIs(name);

					for (LIS lis : lislt) {
						criticalNeoDTO = new CriticalNeoDTO();
						nodename = lis.getName();
						criticalNeoDTO.setName(nodename);
						criticalNeoDTO.setStandname(nodename);
						criticalNeoDTO.setType("化验");

						List<LISCritical> lisCriticals = new ArrayList<>(lis.getLiscritial());

						for (LISCritical lisCritical : lisCriticals) {
							agemin = 0;
							agemax = 0;
							if (lisCritical.getAge().contains(":")) {
								String[] agerange = lisCritical.getAge().split(":");
								agemin = Integer.parseInt(agerange[0]);
								agemax = Integer.parseInt(agerange[1]);
							}

							if (age>=agemin && age<=agemax) {
								NodeNeoDTO nodeNeoDTO = new NodeNeoDTO();

								BigDecimal min = (StringUtil.isNotBlank(lisCritical.getMinval()))
										?(new BigDecimal(lisCritical.getMinval())):null;
								BigDecimal max = (StringUtil.isNotBlank(lisCritical.getMaxval()))
										?(new BigDecimal(lisCritical.getMaxval())):null;

								nodeNeoDTO.setMin(min);
								nodeNeoDTO.setMax(max);
								nodeNeoDTO.setUnit(lisCritical.getUnit());
								nodeNeoDTO.setValType(Integer.parseInt(lisCritical.getRange()));
								criticalNeoDTO.setCritical(nodeNeoDTO);

								lislist.add(criticalNeoDTO);
							}
						}
					}
				}
			}
		}
		catch (Exception ex) {
			ex.printStackTrace();
		}

		return lislist;
	}


	/**
	 * 更新实体属性信息
	 */
	public NeoEntityDTO updateEntity(String name, Map<String, String> props, List<String> keys, LisRepository lisRepository) {
		NeoEntityDTO neoEntityDTO = new NeoEntityDTO();
		neoEntityDTO.setName(name);

		try {
			List<LIS> lisres = lisRepository.findByNameIs(name);

			if (lisres!= null && lisres.size()>0) {
				LIS lis = lisres.get(0);

				for (String key : props.keySet()) {
					switch (key) {
						case "最大值":
							lis.setMaxval(Double.parseDouble(props.get(key)));
							break;
						case "最小值":
							lis.setMinval(Double.parseDouble(props.get(key)));
							break;
						case "单位":
							lis.setUnit(props.get(key));
							break;
					}
				}

				lis = lisRepository.save(lis);

				neoEntityDTO = fillNeoDTO(lis, keys);
			}
		}
		catch (Exception ex) {
			ex.printStackTrace();
		}

		return neoEntityDTO;
	}


	private NeoEntityDTO fillNeoDTO(LIS lis, List<String> keys) {
		NeoEntityDTO neoEntityDTO = new NeoEntityDTO();
		Map<String, String> props = new HashMap<>();
		String val;

		neoEntityDTO.setName(lis.getName());
		for (String key:keys) {
			val = "";
			switch(key) {
				case "最大值":
					val = (lis.getMaxval()==null)?"":String.format("%.2f",lis.getMaxval());
					break;
				case "最小值":
					val = (lis.getMinval()==null)?"":String.format("%.2f", lis.getMinval());
					break;
				case "单位":
					val = lis.getUnit();
					break;
			}
			if (StringUtil.isNotBlank(val)) {
				props.put(key, val);
			}
		}
		neoEntityDTO.setProperty(props);

		return neoEntityDTO;
	}
}

